import { LogInput } from './LogInput';
import { LogLevels } from './logLevels';
import { LogOutput } from './LogOutput';

/**
 * Provides methods for writing logs to in different log-levels. Logs are written in JSON format to stdout using console.log().
 */
class Logger {
  /**
   * This governs which logs will be written. If log level is
   * INFO all logs with INFO and above will be written. So: INFO
   * WARN and ERROR.
   */
  public level: LogLevels = LogLevels.INFO;

  /**
   * Prints an enriched log as JSON to stdout with newline.
   * @param logInput {message: 'This is my message', context: AwsLambdaContext, xRequestId: 'id-from-external-system', something: 'my property', somethingElse: 'another extra property' }
   */
  public debug(logInput: LogInput) {
    let debug = LogLevels.DEBUG;
    if (debug >= this.level) {
      let log = this.createLogJson(logInput, LogLevels.DEBUG);
      console.log(JSON.stringify(log, null, 2));
    }
  }

  /**
   * Prints an enriched log as JSON to stdout with newline.
   * @param logInput {message: 'This is my message', context: AwsLambdaContext, xRequestId: 'id-from-external-system', something: 'my property', somethingElse: 'another extra property' }
   */
  public info(logInput: LogInput) {
    let info = LogLevels.INFO;
    if (info >= this.level) {
      let log = this.createLogJson(logInput, LogLevels.INFO);
      console.log(JSON.stringify(log, null, 2));
    }
  }

  /**
   * Prints an enriched log as JSON to stdout with newline.
   * @param logInput {message: 'This is my message', context: AwsLambdaContext, xRequestId: 'id-from-external-system', something: 'my property', somethingElse: 'another extra property' }
   */
  public warn(logInput: LogInput) {
    let warn = LogLevels.WARN;
    if (warn >= this.level) {
      let log = this.createLogJson(logInput, LogLevels.WARN);
      console.log(JSON.stringify(log, null, 2));
    }
  }

  /**
   * Prints an enriched log as JSON to stdout with newline.
   * @param logInput {message: 'This is my message', context: AwsLambdaContext, xRequestId: 'id-from-external-system', something: 'my property', somethingElse: 'another extra property' }
   */
  public error(logInput: LogInput) {
    let error = LogLevels.ERROR;
    if (error >= this.level) {
      let log = this.createLogJson(logInput, LogLevels.ERROR);
      console.log(JSON.stringify(log, null, 2));
    }
  }

  private createLogJson(logInput: LogInput, level: LogLevels): LogOutput {
    // @ts-ignore
    let logOutput: LogOutput = {};
    logOutput['@timestamp'] = Date.now();
    logOutput.level = LogLevels[level];
    logOutput.message = logInput.message;

    /**
     * By using the lambda context we can easily add more rich logging.
     */
    if (logInput.context) {
      logOutput.awsAccount = logInput.context.invokedFunctionArn ? logInput.context.invokedFunctionArn.split(':')[4] : undefined;
      logOutput.awsRequestId = logInput.context.awsRequestId ? logInput.context.awsRequestId : undefined;
      logOutput.functionName = logInput.context.functionName ? logInput.context.functionName : undefined;
      logOutput.functionVersion = logInput.context.functionVersion ? logInput.context.functionVersion : undefined;
      logOutput.invokedFunctionArn = logInput.context.invokedFunctionArn ? logInput.context.invokedFunctionArn : undefined;
      logOutput.logGroupName = logInput.context.logGroupName ? logInput.context.logGroupName : undefined;
      logOutput.logStreamName = logInput.context.logStreamName ? logInput.context.logStreamName : undefined;
      logOutput.memoryLimitInMB = logInput.context.memoryLimitInMB ? logInput.context.memoryLimitInMB : undefined;
      logOutput.remainingTime = logInput.context.getRemainingTimeInMillis ? logInput.context.getRemainingTimeInMillis() : undefined;
    }

    for (let prop of Object.keys(logInput)) {
      if (prop === 'message' || prop === 'xRequestId' || prop === 'context') { continue; }
      logOutput[prop] = logInput[prop];
    }

    return logOutput;
  }

}

export const log = new Logger();
